@@ -21,28 +21,33 @@ class ExcelConfiguration(ReaderConfiguration):
     a Process Name and Process Type.  The Process is related to the mechanism
     by which source becomes the target (e.g. a Stored Procedure or Query).
 
-    :param str column_sheet: Defaults to "Columns"
-    :param str table_sheet: Defaults to "Tables"
+    :param str bulkEntity_sheet: Defaults to "BulkEntities"
+    :param str updateLineage_sheet: Defaults to "UpdateLineage"
+    :param str columnMapping_sheet: Defaults to "ColumnMapping"
     :param str entityDef_sheet: Defaults to "EntityDefs"
-    :param str entity_source_prefix:
-        Defaults to "source" and represents the prefix of the columns
+    :param str classificationDef_sheet: Defaults to "ClassificationDefs"
+    :param str table_sheet: Defaults to "TablesLineage"
+    :param str column_sheet: Defaults to "FineGrainColumnLineage"
+    :param str source_prefix:
+        Defaults to "Source" and represents the prefix of the columns
         in Excel to be considered related to the source table or column.
-    :param str entity_target_prefix:
-        Defaults to "target" and represents the prefix of the columns
+    :param str target_prefix:
+        Defaults to "Target" and represents the prefix of the columns
         in Excel to be considered related to the target table or column.
-    :param str entity_process_prefix:
-        Defaults to "process" and represents the prefix of the columns
+    :param str process_prefix:
+        Defaults to "Process" and represents the prefix of the columns
         in Excel to be considered related to the table process.
     :param str column_transformation_name:
         Defaults to "transformation" and identifies the column that
         represents the transformation for a specific column.
     """
 
-    def __init__(self, column_sheet="ColumnsLineage",
+    def __init__(self, column_sheet="FineGrainColumnLineage",
                  table_sheet="TablesLineage",
                  entityDef_sheet="EntityDefs", bulkEntity_sheet="BulkEntities",
                  classificationDef_sheet="ClassificationDefs",
                  updateLineage_sheet="UpdateLineage",
+                 columnMapping_sheet="ColumnMapping",
                  **kwargs):
         super().__init__(**kwargs)
         # Required attributes:
@@ -53,6 +58,7 @@ class ExcelConfiguration(ReaderConfiguration):
         self.classificationDef_sheet = classificationDef_sheet
         self.bulkEntity_sheet = bulkEntity_sheet
         self.updateLineage_sheet = updateLineage_sheet
+        self.columnMapping_sheet = columnMapping_sheet
 
 
 class ExcelReader(Reader):
@@ -90,18 +96,25 @@ class ExcelReader(Reader):
 
         return output
 
-    def parse_bulk_entities(self, filepath):
+    def parse_bulk_entities(self, filepath, contacts_func=None):
         """
         Generate a set of entities from an excel template file.
 
         :param str filepath:
             The xlsx file that contains your table and columns.
+        :param function contacts_func:
+            For Azure Purview, a function to be called on each value
+            when you pass in an experts or owners header to json_rows.
+            Leaving it as None will return the exact value passed in
+            to the experts and owners section.
+            It has a built in cache that will prevent redundant calls
+            to your function.
+
         :return: An AtlasTypeDef with entityDefs for the provided rows.
         :rtype: dict(str, list(dict))
         """
         wb = load_workbook(filepath)
-        # A user may omit the entityDef_sheet by providing the
-        # config with None
+
         sheetIsNotPresent = self.config.bulkEntity_sheet not in wb.sheetnames
         if self.config.bulkEntity_sheet and sheetIsNotPresent:
             raise KeyError("The sheet {} was not found".format(
@@ -114,7 +127,7 @@ class ExcelReader(Reader):
             json_bulkEntities = ExcelReader._parse_spreadsheet(
                 bulkEntity_sheet)
             bulkEntities_generated = super().parse_bulk_entities(
-                json_bulkEntities)
+                json_bulkEntities, contacts_func)
             output.update(bulkEntities_generated)
 
         wb.close()
@@ -154,7 +167,7 @@ class ExcelReader(Reader):
         # TODO: Add in classificationDefs and relationshipDefs
         return output
 
-    def parse_column_lineage(self, filepath, atlas_entities, atlas_typedefs, use_column_mapping=False):
+    def parse_finegrain_column_lineage(self, filepath, atlas_entities, atlas_typedefs, use_column_mapping=False):
         """
         Read a given excel file that conforms to the excel atlas template and
         parse the columns into column lineages.
@@ -196,7 +209,7 @@ class ExcelReader(Reader):
         column_sheet = wb[self.config.column_sheet]
         json_columns = ExcelReader._parse_spreadsheet(column_sheet)
 
-        entities = super().parse_column_lineage(
+        entities = super().parse_finegrain_column_lineage(
             json_columns,
             atlas_entities,
             atlas_typedefs,
@@ -239,7 +252,7 @@ class ExcelReader(Reader):
 
         return entities
 
-    def parse_lineages(self, filepath, atlas_typedefs, use_column_mapping=False):
+    def parse_table_finegrain_column_lineages(self, filepath, atlas_typedefs, use_column_mapping=False):
         """
         Read a given excel file that conforms to the excel atlas template and
         parse the tables, processes, and columns into table and column
@@ -274,7 +287,7 @@ class ExcelReader(Reader):
         entities.extend(table_entities)
 
         # Modifies table_entities if use_column_mapping is True
-        column_entities = self.parse_column_lineage(
+        column_entities = self.parse_finegrain_column_lineage(
             filepath,
             table_entities,
             atlas_typedefs,
@@ -323,6 +336,71 @@ class ExcelReader(Reader):
 
         return entities
 
+    def parse_column_mapping(self, filepath):
+        """
+        Read a given excel file that conforms to the excel atlas template and
+        parse the (default) ColumnMapping tab into existing process entities. 
+
+        Assumes these process entities and any referenced entity exists.
+        This will not update the inputs and outputs, it will update name
+        and columnMapping fields.
+
+        :param str filepath:
+            The xlsx file that contains your table and columns.
+        :return:
+            A list of Atlas Process entities representing the spreadsheet's
+            contents.
+        :rtype: list(dict)
+        """
+        wb = load_workbook(filepath)
+
+        entities = []
+
+        if self.config.columnMapping_sheet not in wb.sheetnames:
+            raise KeyError("The sheet {} was not found".format(
+                self.config.columnMapping_sheet))
+
+        # Getting table entities
+        columnMapping_sheet = wb[self.config.columnMapping_sheet]
+        json_sheet = ExcelReader._parse_spreadsheet(columnMapping_sheet)
+        entities = super().parse_column_mapping(json_sheet)
+
+        wb.close()
+
+        return entities
+
+    def parse_update_lineage_with_mappings(self, filepath):
+        """
+        Read a given excel file that conforms to the excel atlas template and
+        parse the (default) UpdateLineage and ColumnMapping tabs into existing process entities. 
+
+        Assumes these process entities and any referenced entity exists.
+
+        :param str filepath:
+            The xlsx file that contains your table and columns.
+        :return:
+            A list of Atlas Process entities representing the spreadsheet's
+            contents.
+        :rtype: list(dict)
+        """
+
+        lineage = self.parse_update_lineage(filepath)
+        mappings = self.parse_column_mapping(filepath)
+        seen_qualifiedNames = {}
+        for working_entity in lineage + mappings:
+            qn = working_entity["attributes"]["qualifiedName"]
+            if qn in seen_qualifiedNames:
+                # If we have seen an entity before check if
+                # the working entity contains a column mapping attribute
+                # if it does update the existing entity
+                if "columnMapping" in working_entity["attributes"]:
+                    seen_qualifiedNames[qn]["attributes"]["columnMapping"] = working_entity["attributes"]["columnMapping"]
+            else:
+                # If we haven't seen it just add the entity to the list
+                seen_qualifiedNames[qn] = working_entity
+
+        return list(seen_qualifiedNames.values())
+
     def parse_classification_defs(self, filepath):
         """
         Read a given excel file that conforms to the excel atlas template and
@@ -347,8 +425,10 @@ class ExcelReader(Reader):
         # Getting classificationDef if the user provided a name of the sheet
         if self.config.classificationDef_sheet:
             classificationDef_sheet = wb[self.config.classificationDef_sheet]
-            json_classificationdefs = ExcelReader._parse_spreadsheet(classificationDef_sheet)
-            classificationDefs_generated = super().parse_classification_defs(json_classificationdefs)
+            json_classificationdefs = ExcelReader._parse_spreadsheet(
+                classificationDef_sheet)
+            classificationDefs_generated = super(
+            ).parse_classification_defs(json_classificationdefs)
             output.update(classificationDefs_generated)
 
         wb.close()
@@ -377,27 +457,100 @@ class ExcelReader(Reader):
                 active_value)
 
     @staticmethod
-    def make_template(filepath):
+    def _replace_header_prefix(headers, prefix_mapping):
+        new_header = []
+        for column in headers:
+            first_token, *remainder = column.split(" ", 1)
+            if first_token in prefix_mapping:
+                new_header.append(
+                    prefix_mapping[first_token]+' '+''.join(remainder))
+            else:
+                new_header.append(column)
+        return new_header
+
+    @staticmethod
+    def make_template(filepath, **kwargs):
         """
         Generate an Excel template file and write it out to the given filepath.
 
         :param str filepath: The file path to store an XLSX file with the
             template Tables and Columns sheets.
+        :param str bulkEntity_sheet: Defaults to "BulkEntities"
+        :param str updateLineage_sheet: Defaults to "UpdateLineage"
+        :param str columnMapping_sheet: Defaults to "ColumnMapping"
+        :param str entityDef_sheet: Defaults to "EntityDefs"
+        :param str classificationDef_sheet: Defaults to "ClassificationDefs"
+        :param str table_sheet: Defaults to "TablesLineage"
+        :param str column_sheet: Defaults to "FineGrainColumnLineage"
+        :param str source_prefix:
+            Defaults to "Source" and represents the prefix of the columns
+            in Excel to be considered related to the source table or column.
+        :param str target_prefix:
+            Defaults to "Target" and represents the prefix of the columns
+            in Excel to be considered related to the target table or column.
+        :param str process_prefix:
+            Defaults to "Process" and represents the prefix of the columns
+            in Excel to be considered related to the table process.
+        :param str column_transformation_name:
+            Defaults to "transformation" and identifies the column that
+            represents the transformation for a specific column.
         """
         wb = Workbook()
-        columnsSheet = wb.active
-        columnsSheet.title = "ColumnsLineage"
-        tablesSheet = wb.create_sheet("TablesLineage")
-        entityDefsSheet = wb.create_sheet("EntityDefs")
-        classificationDefsSheet = wb.create_sheet("ClassificationDefs")
-        bulkEntitiesSheet = wb.create_sheet("BulkEntities")
-        updateLineageSheet = wb.create_sheet("UpdateLineage")
+        bulkEntitiesSheet = wb.active
+        bulkEntitiesSheet.title = kwargs.get(
+            "bulkEntity_sheet", "BulkEntities")
+        updateLineageSheet = wb.create_sheet(
+            kwargs.get("updateLineage_sheet", "UpdateLineage"))
+        columnMappingSheet = wb.create_sheet(
+            kwargs.get("columnMapping_sheet", "ColumnMapping"))
+        entityDefsSheet = wb.create_sheet(
+            kwargs.get("entityDef_sheet", "EntityDefs"))
+        classificationDefsSheet = wb.create_sheet(kwargs.get(
+            "classificationDef_sheet", "ClassificationDefs"))
+        tablesSheet = wb.create_sheet(
+            kwargs.get("table_sheet", "TablesLineage"))
+        columnsSheet = wb.create_sheet(kwargs.get(
+            "column_sheet", "FineGrainColumnLineage"))
+
+        # Supporting changing the default headers on select pages
+        header_changes = {}
+        if "source_prefix" in kwargs:
+            header_changes["Source"] = kwargs["source_prefix"]
+        if "target_prefix" in kwargs:
+            header_changes["Target"] = kwargs["target_prefix"]
+        if "process_prefix" in kwargs:
+            header_changes["Process"] = kwargs["process_prefix"]
+        if "column_transformation_name" in kwargs:
+            header_changes["transformation"] = kwargs["column_transformation_name"]
+
+        if header_changes:
+            FineGrainColumnLineageHeaders = ExcelReader._replace_header_prefix(
+                Reader.TEMPLATE_HEADERS["FineGrainColumnLineage"],
+                header_changes
+            )
+            TablesLineageHeaders = ExcelReader._replace_header_prefix(
+                Reader.TEMPLATE_HEADERS["TablesLineage"],
+                header_changes
+            )
+            UpdateLineageHeaders = ExcelReader._replace_header_prefix(
+                Reader.TEMPLATE_HEADERS["UpdateLineage"],
+                header_changes
+            )
+            ColumnMappingHeaders = ExcelReader._replace_header_prefix(
+                Reader.TEMPLATE_HEADERS["ColumnMapping"],
+                header_changes
+            )
+        else:
+            FineGrainColumnLineageHeaders = Reader.TEMPLATE_HEADERS["FineGrainColumnLineage"]
+            TablesLineageHeaders = Reader.TEMPLATE_HEADERS["TablesLineage"]
+            UpdateLineageHeaders = Reader.TEMPLATE_HEADERS["UpdateLineage"]
+            ColumnMappingHeaders = Reader.TEMPLATE_HEADERS["ColumnMapping"]
 
         ExcelReader._update_sheet_headers(
-            Reader.TEMPLATE_HEADERS["ColumnsLineage"], columnsSheet
+            FineGrainColumnLineageHeaders, columnsSheet
         )
         ExcelReader._update_sheet_headers(
-            Reader.TEMPLATE_HEADERS["TablesLineage"], tablesSheet
+            TablesLineageHeaders, tablesSheet
         )
         ExcelReader._update_sheet_headers(
             Reader.TEMPLATE_HEADERS["EntityDefs"], entityDefsSheet
@@ -409,7 +562,10 @@ class ExcelReader(Reader):
             Reader.TEMPLATE_HEADERS["BulkEntities"], bulkEntitiesSheet
         )
         ExcelReader._update_sheet_headers(
-            Reader.TEMPLATE_HEADERS["UpdateLineage"], updateLineageSheet
+            UpdateLineageHeaders, updateLineageSheet
+        )
+        ExcelReader._update_sheet_headers(
+            ColumnMappingHeaders, columnMappingSheet
         )
 
         wb.save(filepath)
