@@ -58,8 +58,9 @@ def column_lineage_scaffold(datasource,
         name=src_table_columns_typeName,
         attributeDefs=table_column_relationship_attributes,
         relationshipCategory="COMPOSITION",
-        endDef1=typedef.ParentEndDef(name="columns",typeName=table_entity.name),
-        endDef2=typedef.ChildEndDef(name="table",typeName=column_entity.name)
+        endDef1=typedef.ParentEndDef(
+            name="columns", typeName=table_entity.name),
+        endDef2=typedef.ChildEndDef(name="table", typeName=column_entity.name)
     )
 
     # Define {datasource}_column_lineage
@@ -100,11 +101,11 @@ def column_lineage_scaffold(datasource,
         relationshipCategory="COMPOSITION",
         attributeDefs=table_process_column_lineage_relationship_attributes,
         endDef1=typedef.ChildEndDef(name="query",
-            typeName=column_lineage_process_entity.name,
-            isLegacyAttribute=True),
+                                    typeName=column_lineage_process_entity.name,
+                                    isLegacyAttribute=True),
         endDef2=typedef.ParentEndDef(name="columnLineages",
-            typeName=table_process_entity.name,
-            isLegacyAttribute=False)
+                                     typeName=table_process_entity.name,
+                                     isLegacyAttribute=False)
     )
 
     # Output composite entity
